﻿using Nai.Hardware.IOBoard.NetStandard;
using System;
using System.Threading;


namespace NaiStudioIOBoard_CSharpConsoleDemo
{
    internal class Program
    {
        static void Main(string[] args)
        {
            BasicExample();

            AdvancedExample();
        }

        private static void BasicExample()
        {
            Console.Clear();
            Console.WriteLine("基础的例子：读取和写入");
            // 通过“信号板实用工具”的刷新按钮来查看 portName，或者通过设备管理器的“端口（COM和LPT）”查看
            string portName = "COM6";
            var board = new DigitalIOBoard(portName, 8, 8);
            try
            {
                board.Connect();
            }
            catch (Exception)
            {
                Console.WriteLine($"通过 {portName} 连接信号板失败，退出");
                return;
            }

            // 读取一次所有信号值到board实例内部存储，用于后面调用 ToggleOne 方法；
            // Toggle 方法做的事情：置反某个输出信号的值 -- 从on置off，或从off置on；所以它需要当前信号的最新值
            board.ReadAll();
            for (ushort i = 0; i < board.Outputs.Length; i++)
            {
                Console.WriteLine($"置反输出通道 {i} 的值...");
                board.ToggleOne(i); // for outputs only
                Thread.Sleep(200);
            }

            // 读取和打印所有通道的值
            board.ReadAll();
            for (int i = 0; i < board.Inputs.Count; i++) // Inputs 的 Count 属性，是sdk v1.24.304.1 新增的
            {
                var value = board.Inputs[i];
                Console.WriteLine($"当前输入通道 {i} 的值 = {value}");
            }
            for (ushort i = 0; i < board.Outputs.Length; i++)
            {
                var value = board.Outputs[i];
                Console.WriteLine($"当前输出通道 {i} 的值 = {value}");
            }

            // 把板卡所有输出置零
            for (ushort i = 0; i < board.Outputs.Length; i++)
            {
                board.WriteOne(i, false); // for outputs only
                Thread.Sleep(200);
            }
            // 最后，断开板卡连接，回收资源
            board.Dispose();
            Console.WriteLine("断开了信号板。程序将退出");
            Thread.Sleep(2000);
            // end method
        }


        static volatile bool exitFlag;
        static void AdvancedExample()
        {
            Console.Clear();
            Console.WriteLine("进阶的例子：监控输入通道的上升沿或下降沿；\r\n使用 ctrl + c 来退出");
            Console.CancelKeyPress += Console_CancelKeyPress;
            // 通过“信号板实用工具”的刷新按钮来查看 portName，或者通过设备管理器的“端口（COM和LPT）”查看
            string portName = "COM6";
            var board = new DigitalIOBoard(portName, 8, 8);
            try
            {
                board.Connect();
            }
            catch (Exception)
            {
                Console.WriteLine($"通过 {portName} 连接信号板失败，退出");
                return;
            }

            var risingEdgeDetector = new Nai.Utils.EdgeDetector<ushort>(expectValue: 1, initialValue: 0);
            var fallingEdgeDetector = new Nai.Utils.EdgeDetector<ushort>(expectValue: 0, initialValue: 1);
            int inputChannelIndex = 0; // 假设监控第一个（输入）通道
            while (!exitFlag)
            {
                board.ReadAll();
                var ch1 = board.Inputs[inputChannelIndex];
                if (risingEdgeDetector.IsEdgeDetected(ch1))
                {
                    Console.WriteLine($"检测到输入通道 {inputChannelIndex} 的上升沿");
                }
                if (fallingEdgeDetector.IsEdgeDetected(ch1))
                {
                    Console.WriteLine($"检测到输入通道 {inputChannelIndex} 的下降沿");
                }
                Thread.Sleep(40);
            }
            // 最后，断开板卡连接，回收资源
            board.Dispose();
            Console.WriteLine("断开了信号板。程序将退出");
            Thread.Sleep(2000);
            // end method
        }

        private static void Console_CancelKeyPress(object sender, ConsoleCancelEventArgs e)
        {
            exitFlag = true;
        }
    }
}
